/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.rdb.adapter;

import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapterException;

/**
 * @author S.Chassande-Barrioz, J.Camilleri
 */
public class Db2Adapter extends BasicRdbAdapter {

	public Db2Adapter() {
		super("db2");
	}

	public String getSqlType(int typeCode, boolean usedInPK, int size, int scale) throws RdbAdapterException {
		switch (typeCode) {
		// ALL COLUMN IN PK SHOULD BE DECLARED NOT NULL
		case TYPECODE_STRING:
			return (usedInPK
					? "VARCHAR(" + size + ") NOT NULL"
					: "VARCHAR(" + size + ")");
		case TYPECODE_BOOLEAN:
		case TYPECODE_OBJBOOLEAN:
			return (usedInPK
					? "INTEGER NOT NULL"
					: "INTEGER");
		case TYPECODE_CHAR:
		case TYPECODE_OBJCHAR:
			return (usedInPK
					? "CHAR NOT NULL"
					: "CHAR");
		case TYPECODE_BYTE:
		case TYPECODE_OBJBYTE:
			return (usedInPK
					? "SMALLINT NOT NULL"
					: "SMALLINT");
		case TYPECODE_SHORT:
		case TYPECODE_OBJSHORT:
			return (usedInPK
					? "SMALLINT NOT NULL"
					: "SMALLINT");
		case TYPECODE_INT:
		case TYPECODE_OBJINT:
			return (usedInPK
					? "INTEGER NOT NULL"
					: "INTEGER");
		case TYPECODE_LONG:
		case TYPECODE_OBJLONG:
			return (usedInPK
					? "BIGINT NOT NULL"
					: "BIGINT");
		case TYPECODE_FLOAT:
		case TYPECODE_OBJFLOAT:
			return (usedInPK
					? "REAL NOT NULL"
					: "REAL");
		case TYPECODE_DOUBLE:
		case TYPECODE_OBJDOUBLE:
			return (usedInPK
					? "DOUBLE NOT NULL"
					: "DOUBLE");
		case TYPECODE_DATE:
			return (usedInPK
					? "DATE NOT NULL"
					: "DATE");
		case TYPECODE_CHARARRAY:
			return (usedInPK
					? "VARCHAR NOT NULL"
					: "VARCHAR");
		case TYPECODE_BYTEARRAY:
			return (usedInPK
					? "VARCHAR(" + size + ") FOR BIT DATA NOT NULL"
					: "VARCHAR(" + size + ") FOR BIT DATA");
		case TYPECODE_SERIALIZED:
			return (usedInPK
					? "VARCHAR(" + size + ") FOR BIT DATA NOT NULL"
					: "VARCHAR(" + size + ") FOR BIT DATA");
		case TYPECODE_BIGINTEGER:
		case TYPECODE_BIGDECIMAL:
			String end = (usedInPK ? " NOT NULL" : "");
			if (size != NOSIZE) {
				if (scale != NOSIZE) {
					return "DECIMAL(" + size + ", " + scale + ")" + end;
				} else {
					return "DECIMAL(" + size + ")"  + end;
				}
			} else {
				return "DECIMAL" + end;
			}
		default:
			return super.getSqlType(typeCode, usedInPK, size, scale);
		}
	}

    /**
     * OL: I'm not sure of this, i didn't manage to have the exact syntax for
     * column alaiases with DB2 :o(
     * @param alias
     * @param sb
     */
    public void writeColumnAlias(String alias, StringBuffer sb) {
        sb.append(" \"" + alias + "\" ");
    }
    public String getColumnAliasExpr(String aliasName) {
        return " \"" + aliasName +"\" ";
    }

}
