/**
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.jorm.mapper.rdb.adapter.api;

/**
 * The supported type in the adapter.
 *
 * @author S.Chassande-Barrioz
 */
public interface TypeAdapter {
	/**
	 * Defines the boolean type.
	 * If you modify this value, beware about the jorm type
	 */
	short TYPECODE_BOOLEAN 		= 0;

	/**
	 * Defines the char type.
	 * If you modify this value, beware about the jorm type
	 */
	short TYPECODE_CHAR 		= 1;

	/**
	 * Defines the byte type.
	 * If you modify this value, beware about the jorm type
	 */
	short TYPECODE_BYTE 		= 2;

	/**
	 * Defines the short type.
	 * If you modify this value, beware about the jorm type
	 */
	short TYPECODE_SHORT 		= 3;

	/**
	 * Defines the int type.
	 * If you modify this value, beware about the jorm type
	 */
	short TYPECODE_INT 			= 4;

	/**
	 * Defines the long type.
	 * If you modify this value, beware about the jorm type
	 */
	short TYPECODE_LONG 		= 5;

	/**
	 * Defines the float type.
	 * If you modify this value, beware about the jorm type
	 */
	short TYPECODE_FLOAT 		= 6;

	/**
	 * Defines the double type.
	 * If you modify this value, beware about the jorm type
	 */
	short TYPECODE_DOUBLE 		= 7;

	/**
	 * Defines the java.lang.Boolean type.
	 * If you modify this value, beware about the jorm type
	 */
	short TYPECODE_OBJBOOLEAN 	= 8;

	/**
	 * Defines the java.lang.Boolean type.
	 * If you modify this value, beware about the jorm type
	 */
	short TYPECODE_OBJCHAR 		= 9;

	/**
	 * Defines the java.lang.Character type.
	 * If you modify this value, beware about the jorm type
	 */
	short TYPECODE_OBJBYTE 		= 10;

	/**
	 * Defines the java.lang.Short type.
	 * If you modify this value, beware about the jorm type
	 */
	short TYPECODE_OBJSHORT 	= 11;

	/**
	 * Defines the java.lang.Integer type.
	 * If you modify this value, beware about the jorm type
	 */
	short TYPECODE_OBJINT 		= 12;

	/**
	 * Defines the java.lang.Long type.
	 * If you modify this value, beware about the jorm type
	 */
	short TYPECODE_OBJLONG 		= 13;

	/**
	 * Defines the java.lang.Float type.
	 * If you modify this value, beware about the jorm type
	 */
	short TYPECODE_OBJFLOAT 	= 14;

	/**
	 * Defines the java.lang.Double type.
	 * If you modify this value, beware about the jorm type
	 */
	short TYPECODE_OBJDOUBLE 	= 15;

	/**
	 * Defines the java.lang.String type.
	 * If you modify this value, beware about the jorm type
	 */
	short TYPECODE_STRING 		= 16;

	/**
	 * Defines the java.util.Date type.
	 * If you modify this value, beware about the jorm type
	 */
	short TYPECODE_DATE 		= 17;

	/**
	 * Defines the char[] type.
	 * If you modify this value, beware about the jorm type
	 */
	short TYPECODE_CHARARRAY 	= 18;

	/**
	 * Defines the byte[] type.
	 * If you modify this value, beware about the jorm type
	 */
	short TYPECODE_BYTEARRAY 	= 19;

	/**
	 * Defines the java.io.Serializable type.
	 * If you modify this value, beware about the jorm type
	 */
	short TYPECODE_SERIALIZED 	= 20;

	/**
	 * Defines the java.util.BigInteger type.
	 * If you modify this value, beware about the jorm type
	 */
	short TYPECODE_BIGINTEGER 	= 21;

	/**
	 * Defines the java.util.BigDecimal type.
	 * If you modify this value, beware about the jorm type
	 */
	short TYPECODE_BIGDECIMAL 	= 22;

	/**
	 * the name of the types
	 * TYPE_NAMES[TYPECODE_XXX]
	 */
	String[] TYPE_NAMES = {
		"boolean",
		"char",
		"byte",
		"short",
		"int",
		"long",
		"float",
		"double",
		"java.lang.Boolean",
		"java.lang.Character",
		"java.lang.Byte",
		"java.lang.Short",
		"java.lang.Integer",
		"java.lang.Long",
		"java.lang.Float",
		"java.lang.Double",
		"java.lang.String",
		"java.util.Date",
		"char[]",
		"byte[]",
		"java.io.Serialized",
		"java.math.BigInteger",
		"java.math.BigDecimal"
	};

	/**
	 * This method returns the SQL type linked to the java type
	 * @param typeCode is the type code of the java type
	 * @param usedInPK indicates if the type has to be used into a Primary key
	 * @param size can indicates the expected size of the type. If it equals to
	 * NO_SIZE that means no size is expected. This parameter can be used for
	 * float, BigXXX numbers, String,...
	 * @param scale can indicates the expected scale of the type. If it equals to
	 * NO_SIZE that means no scale is expected.
	 */
	String getSqlType(int typeCode, boolean usedInPK, int size, int scale)
		throws RdbAdapterException;

	/**
	 * This method returns the SQL type code linked to the java type
	 * specified in parameter.
	 * @return An int constant defined in the java.sql.Types class
	 */
	int getSqlTypeCode(int typeCode) throws RdbAdapterException;

	/**
	 * This method returns the SQL type code linked to the java type
	 * specified in parameter.
	 * @return An int constant defined in the java.sql.Types class
	 */
	int getSqlTypeCode(int typeCode, String sqlColumnType) throws RdbAdapterException;

}
